import ctypes
import os
from collections import defaultdict


absolute_path = os.path.dirname(__file__)
absolute_path_ubuntu = os.path.join(absolute_path, "lib_ubuntu", "library.so")
absolute_path_windows32 = os.path.join(absolute_path, "lib_windows", 'x86', 'Release',  "lib_windows.dll")
absolute_path_windows64 = os.path.join(absolute_path, "lib_windows", 'x64', 'Release',  "lib_windows.dll")

# # Load the shared library
try:
    lib = ctypes.CDLL(absolute_path_ubuntu)
except:
    try:
        lib = ctypes.CDLL(absolute_path_windows32)
    except :
        lib = ctypes.CDLL(absolute_path_windows64)

get_community_assignment = lib.get_community_assignment
get_community_assignment.restype = ctypes.POINTER(ctypes.c_int)

def get_community(nodes, edges):
    nodeNum = 0
    mp_nodes={}
    new_nodes=[n for n in range(len(nodes))]
    new_edges=[]
    for n in nodes:
        mp_nodes[n] = nodeNum
        nodeNum+=1
    
    for e in edges:
        new_edges.append([ mp_nodes[e[0]], mp_nodes[e[1]] ])

    arr = (ctypes.c_int * (len(new_edges)*2))()
    for i,[u,v] in enumerate(new_edges):
        arr[(i*2)] = u
        arr[i*2+1] = v
    
    nrows = len(new_edges)
    nodeCount = len(new_nodes)

    res = get_community_assignment(
        arr,
        ctypes.c_int(nrows),
        ctypes.c_int(nodeCount))
    
    
    res = [res[i] for i in range(len(nodes))]
    com = defaultdict(list)
    for n in nodes:
        com[res[mp_nodes[n]]].append(n)
    final_com = []
    for x in com :
        final_com.append(com[x])
    
    return final_com







from . import Exception

def Py1dList(data):
    """Check if its an 1d array List"""
    if not isinstance(data, list):
        raise Exception.NotListException(data)

def Py2dList(data):
    """Check if its an 2d array List"""
    if not isinstance(data, list) or any(not isinstance(sublist, list) for sublist in data):
        raise Exception.Not2DListException(data)

def EdgeArray(edges):
    """check if its an array of 2 columns"""
    for e in edges:
        if (len(e)!=2):
            raise Exception.NotAnEdgeException(e)

def EdgeExistInNodes(nodes, edges):
    """Check if Edge exist in node list"""
    for u,v in edges:
        if u not in nodes:
            raise Exception.NodeNotExistException(u)
        if v not in nodes:
            raise Exception.NodeNotExistException(v)

def isGraphCreated(nodes,edges):
    """Check if Graph created of not"""
    if(nodes == None):
        raise Exception.GraphNotCreatedException()
    if(edges== None):
        raise Exception.GraphNotCreatedException()


from setuptools import setup, Extension
from age_cda import VERSION

with open("README.md", "r", encoding='utf8') as fh:
    long_description = fh.read()

setup(
    name             = 'age_cda',
    version          = VERSION.VERSION,
    description      = 'Detection of Community by maximizing modularity',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author           = 'Moontasir Mahmood',
    author_email     = 'moontasir042@gmail.com',
    url              = 'https://github.com/Munmud/Community-Detection-Modularity',
    license          = 'Apache2.0',
    install_requires = [],
    packages         = ['age_cda'],
    package_data     = {
        'age_cda': 
                    [   
                        'lib_ubuntu/*.so', 
                        'lib_windows/x64/Release/*.dll', 
                        'lib_windows/x86/Release/*.dll'
                    ]
    },
    keywords         = [
        'Community-Detection', 
        'Modularity', 
        'Reichardt and Bornholdt',
        'Newman', 
        'partition network', 
        'k means cluster'
    ],
    python_requires  = '>=3.9',
    classifiers      = [
        'Programming Language :: Python :: 3.9'
    ]
)


from . import Check
from . import Lib

class Graph:
    def __init__(self):
        self.nodes = None
        self.edges = None
    
    def createGraph(self, nodes, edges):
        Check.Py1dList(nodes)
        Check.Py2dList(edges)
        Check.EdgeArray(edges)
        Check.EdgeExistInNodes(nodes,edges)
        self.nodes = nodes
        self.edges = edges

    def __str__(self):
        return f"Graph with nodes : {self.nodes} edges : ({self.edges})"
    
    def get_community(self):
        Check.isGraphCreated(self.nodes, self.edges)
        return Lib.get_community(self.nodes,self.edges)

